
/*
	Easy remote control for SoundPlay.
	Copyright 1999 Marco Nelissen (marcone@xs4all.nl)
*/

#include <Application.h>
#include <Roster.h>
#include <stdio.h>
#include <string>

static const char *SOUNDPLAY_SIGNATURE="application/x-vnd.marcone-soundplay";

BMessenger RemoteTrack(int32 index)
{
	BMessage question(B_GET_PROPERTY);
	question.AddSpecifier("Messenger");  // a track is a BHandler, and therefore supports the handler scripting suite
	question.AddSpecifier("track",index);
	BMessage answer;
	BMessenger remotetrack;

	if(B_OK == BMessenger(SOUNDPLAY_SIGNATURE).SendMessage(&question, &answer,2E6,2E6))
	{
		int32 error;
		if(B_OK == answer.FindInt32("error",&error) && error == B_OK && answer.HasMessenger("result"))
			answer.FindMessenger("result",&remotetrack);
	}
	return remotetrack;
}


void DoCommand(int32 what)
{
	switch(what)
	{
		case 'stop':
			{
				for(int i=0;;i++)
				{
					BMessenger track = RemoteTrack(i);
					if(!track.IsValid())
						break;
					BMessage command(B_SET_PROPERTY);
					command.AddFloat("data",0.0);
					command.AddSpecifier("pitch");
					track.SendMessage(&command, (BHandler*)NULL, 2E6);
				}
			}
			break;

		case 'play':
			{
				BMessenger sp(SOUNDPLAY_SIGNATURE);
				if(!sp.IsValid())
				{
					BRoster().Launch(SOUNDPLAY_SIGNATURE);
					
					bigtime_t maxtime = system_time() + 10000000;
					while(system_time() < maxtime)
					{
						if(BMessenger(SOUNDPLAY_SIGNATURE).IsValid())
							break;
						snooze(500000);
					}
				}

				for(int i=0;;i++)
				{
					BMessenger track = RemoteTrack(i);
					if(!track.IsValid())
						break;
					BMessage command(B_SET_PROPERTY);
					command.AddFloat("data",1.0);
					command.AddSpecifier("pitch");
					track.SendMessage(&command, (BHandler*)NULL, 2E6);
				}
			}
			break;

		case 'prev':
			{
				for(int i=0;;i++)
				{
					BMessenger track = RemoteTrack(i);
					if(!track.IsValid())
						break;
					BMessage command(B_EXECUTE_PROPERTY);
					command.AddSpecifier("previous");
					track.SendMessage(&command, (BHandler*)NULL, 2E6);
				}
			}
			break;

		case 'next':
			{
				for(int i=0;;i++)
				{
					BMessenger track = RemoteTrack(i);
					if(!track.IsValid())
						break;
					BMessage command(B_EXECUTE_PROPERTY);
					command.AddSpecifier("next");
					track.SendMessage(&command, (BHandler*)NULL, 2E6);
				}
			}
			break;

		case 'vol-':
			{
				for(int i=0;;i++)
				{
					BMessenger track = RemoteTrack(i);
					if(!track.IsValid())
						break;
					BMessage command(B_GET_PROPERTY);
					command.AddSpecifier("volume");
					BMessage reply;
					track.SendMessage(&command, &reply, 2E6, 2E6);
					float volume;
					if(B_OK == reply.FindFloat("result",&volume))
					{
						volume -= 0.02;
						if(volume<0.0)
							volume = 0.0;
						BMessage command(B_SET_PROPERTY);
						command.AddFloat("data",volume);
						command.AddSpecifier("volume");
						track.SendMessage(&command, (BHandler*)NULL, 2E6);
					}
				}
			}
			break;

		case 'vol+':
			{
				for(int i=0;;i++)
				{
					BMessenger track = RemoteTrack(i);
					if(!track.IsValid())
						break;
					BMessage command(B_GET_PROPERTY);
					command.AddSpecifier("volume");
					BMessage reply;
					track.SendMessage(&command, &reply, 2E6, 2E6);
					float volume;
					if(B_OK == reply.FindFloat("result",&volume))
					{
						volume += 0.02;
						if(volume>2.0)
							volume = 2.0;
						BMessage command(B_SET_PROPERTY);
						command.AddFloat("data",volume);
						command.AddSpecifier("volume");
						track.SendMessage(&command, (BHandler*)NULL, 2E6);
					}
				}
			}
			break;

		case 'mute':
			{
				for(int i=0;;i++)
				{
					BMessenger track = RemoteTrack(i);
					if(!track.IsValid())
						break;
					BMessage command(B_GET_PROPERTY);
					command.AddSpecifier("volume");
					BMessage reply;
					track.SendMessage(&command, &reply, 2E6, 2E6);
					float volume;
					if(B_OK == reply.FindFloat("result",&volume))
					{
						if(volume > 0.03)
							volume = 0.0;
						else
							volume = 1.0;
						BMessage command(B_SET_PROPERTY);
						command.AddFloat("data",volume);
						command.AddSpecifier("volume");
						track.SendMessage(&command, (BHandler*)NULL, 2E6);
					}
				}
			}
			break;
	}
}

int main(int argc, char *argv[])
{
	bool displayusage=false;
	if(argc<=1)
		displayusage=true;
	while(--argc > 0)
	{
		char *cmd = *++argv;	
		if(strcmp(cmd,"play") == 0)
			DoCommand('play');
		else if(strcmp(cmd,"stop") == 0)
			DoCommand('stop');
		else if(strcmp(cmd,"next") == 0)
			DoCommand('next');
		else if(strcmp(cmd,"prev") == 0)
			DoCommand('prev');
		else if(strcmp(cmd,"prev") == 0)
			DoCommand('prev');
		else if(strcmp(cmd,"vol-") == 0)
			DoCommand('vol-');
		else if(strcmp(cmd,"vol+") == 0)
			DoCommand('vol+');
		else if(strcmp(cmd,"mute") == 0)
			DoCommand('mute');
		else
			displayusage=true;
	}
	if(displayusage)
	{
		printf(	"Usage: spctrl <command>\n"
				"Where <command> is one or more of:\n"
				"play, stop, next, prev, vol-, vol+, mute\n");
	}
}

